
-- Based on Settings.lua by Pengoros



Settings = class();

-- encode numbers as strings if necessary for the client to save properly
function Settings:EncodeNumbers(object)
    local lookup_table = {};
    local function _encode(object)
        if (type(object) == "number") then
            return ("<num>%f</num>"):format(object);
        elseif (type(object) ~= "table") then
            return object;
        elseif (lookup_table[object]) then
            return lookup_table[object];
        end
        local new_table = {};
        lookup_table[object] = new_table;
        for index, value in pairs(object) do
            new_table[_encode(index)] = _encode(value);
        end
        return setmetatable(new_table, getmetatable(object));
    end
    return _encode(object);
end

-- decode numbers encoded as strings
function Settings:DecodeNumbers(object)
    local lookup_table = {};
    local function _decode(object)
        if (type(object) == "number") then
        	return object;
        elseif (type(object) == "string") then
            local match = object:match("<num>(\-?%d+).%d+</num>");
            -- Kiki.Commons.Trace("Match="..object.."=".. (match or "nil") .. " tonumber=" .. (tonumber(match) or "n/a") ); 
            if (match ~= nil) then
                return tonumber(match);
            else
                return object;
            end
        elseif (type(object) ~= "table") then
            return object;
        elseif lookup_table[object] then
            return lookup_table[object];
        end
        local new_table = {};
        lookup_table[object] = new_table;
        for index, value in pairs(object) do
        	local idx = _decode(index);
        	if idx ~= nil then
            	new_table[idx] = _decode(value);
            else
            	Kiki.Commons.Trace("Warning: Decodenumbers encounters nil index index=".. dump(index) .. " value=".. dump(value)); 
            end
        end
        return setmetatable(new_table, getmetatable(object));
    end
    return _decode(object);
end

function Settings:Constructor(name,default)

	function _gss()
		return Turbine.PluginData.Load(Turbine.DataScope.Server, name);
	end
	local isOk, saved = pcall(_gss);

	if not isOk or type(saved) ~= "table" then
		if saved == nil then
			Kiki.Commons.Trace("Using default settings for ".. name);
		else
			Kiki.Commons.Trace("Error loading Settings, using default settings");
		end
		saved = deepcopy(default);
	end
    saved = self:DecodeNumbers(saved);

 	for i,v in pairs(saved) do
 		self[i] = v;
 	end


end

function Settings:Save(name)
    local object = Settings:EncodeNumbers(self);
    Turbine.PluginData.Save(Turbine.DataScope.Server, name, object);
    -- Kiki.Commons.Trace("Settings: "..name.." saved");
end




